require 'rubygems'
require 'sinatra'
require 'redis'
require 'json'

configure do
  DB = Redis.new
  SELF_URL = 'http://localhost:5000'
end

get '/' do
  @cookie = request.cookies['2short']

  if @cookie.nil? || DB.get('cookie_ids').nil?
    myid = DB.incr('cookie_ids')
    response.set_cookie('2short', myid)
  elsif DB.keys("slugs_by_cookie_#{@cookie}")
    myid = @cookie
  end

  slug_list = "slugs_by_cookie_#{myid}"
  @slugs = DB.lrange(slug_list, 0, -1)
  @slug_data = {}
  @slugs.each do |slug|
    @slug_data[slug] = JSON.parse(DB.get(slug))
  end

  haml :'index'
end

get '/style.css' do
  headers 'Content-Type' => 'text/css; charset=utf-8'
  sass :style
end

post '/shorten' do
  if params[:url] =~ /^https?:/
    @cookie = request.cookies['2short']
    @slug_list = "slugs_by_cookie_#{@cookie}"
    slug = shorten(params[:url], params[:sticky])
    host = Sinatra::Application.bind

    "#{SELF_URL}/#{slug}"
  else
    reject_blank(params[:url])
  end
end

get '/favicon.ico' do
  halt(404)
end

get '/:slug' do |slug|
  if url = url_for(slug)
    redirect(url)
  else
    halt(404)
  end
end

helpers do
  def reject_blank(url)
    redirect('/') unless url.size > 0
  end

  def shorten(url, sticky = false)
    slug_exists = true
    chars = 'abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ1234567890'

    while slug_exists == true
      slug = ''
      8.times { slug << chars[rand(chars.size)] }

      slug_exists = DB.exists(slug)
    end

    DB.set(slug, {:clicks => 0, :url => url, :date => Time.now, :sticky => sticky}.to_json)
    DB.lpush(@slug_list, slug)
    slug
  end

  def url_for(slug)
    if json = DB.get(slug)
      slug_data = JSON.parse(json)
      slug_data['clicks'] += 1
      DB.set(slug, slug_data.to_json)
      slug_data['url']
    else
      return nil
    end
  end

  def truncate(string, len = 20)
    if string
      (string.length > len ? string[0...len] + '...' : string).to_s
    end
  end
end
